function mainLSI(CoOccurenceMatrixFileName,numberOfDocuments,listOfIssueIDsFileName,k,prefixFileNameSimilarity,outputFolderLSISimilarities)
%mainLSI Implementation of the Latent Semantic Indexing (LSI) technique.
%	mainLSI computes the similarities between user queries (appended at the
%       end of the corpus) and a corpus of documents (e.g., methods, classes)
%
%	mainLSI(CoOccurenceMatrixFileName,numberOfDocuments,listOfIssueIDsFileName,k,outputFolderLSISimilarities)
%       CoOccurenceMatrixFileName - the file name of the term by document
%           matrix with numberOfDocuments+numberOfQueries rows and
%           numberOfTerms columns (the values in the matrix are separated
%           by spaces, and each document is on its own line). The first
%           numberOfDocuments rows represent the documents (e.g., methods),
%           while the remaining numberOfQueries rows represent the queries
%       numberOfDocuments - the number of documents (e.g., methods) which
%           represent the first rows of the CoOccurenceMatrixFileName matrix
%       listOfIssueIDsFileName - the file name containing the list of
%           issueIDs associated with the queries from the last
%           numberOfQueries rows of the CoOccurenceMatrixFileName matrix.
%           The issueID from position i of the file listOfIssueIDsFileName
%           is associated with query i from row numberOfDocuments+i from
%           the CoOccurenceMatrixFileName
%       k - the dimensionality reduction factor of LSI (e.g., 300)
%       prefixFileNameSimilarity - the prefix of the LSI similarities file
%           that will embed some information about the configuration of LSI
%           (e.g., "LSI_k=300_")
%       outputFolderLSISimilarities - the folder name where the similarities
%           between each query and all the documents of the corpus will be
%           stored. One file with similarities will be generated for each
%           query and will have the name prefixFileNameSimilarity+issueID."sim". The
%           content of the file will consist of 2-tuples (one per line) in
%           the format "documentID cosineSimilarityBetweenQueryAndDocumentID"
%   example:
%       mainLSI('TestCase/CoOccurenceMatrix.txt',4,'TestCase/listOfIssueIDs.txt',3,'LSI_k=3_','TestCase/OutputLSISimilarities/')
%
%   version 1 (2012-10-24)
%       - initial release
%
%   Copyright 2012 SEMERU Research Group http://www.cs.wm.edu/semeru/

    %size of CoOccurenceMatrix should be (numberOfDocuments+numberOfQueries) x numberOfTerms
    CoOccurenceMatrix=load(CoOccurenceMatrixFileName);

	%load the issue ids; textread returns a cell, so each element (i.e.,
	%each issueID) needs to be converted to char
    listOfIssueIDs=textread(listOfIssueIDsFileName,'%s');
    
    numberOfQueries=size(listOfIssueIDs,1);

	%check if the number of queries is the same with the number of queries
	%in the matrix
    if numberOfQueries~=(size(CoOccurenceMatrix,1)-numberOfDocuments)
        error('numberOfQueries~=(size(CoOccurenceMatrix,1)-numberOfDocuments)');
    end;
    
    %size of weightedMatrix should be (numberOfDocuments+numberOfQueries) x numberOfTerms
    [weightedMatrix]=calculateTFIDF(CoOccurenceMatrix);
    
    %the sparseWeightedMatrix (required for svds) should have the size
    %numberOfTerms x (numberOfDocuments+numberOfQueries)
    sparseWeightedMatrix=sparse(weightedMatrix');
    
    [T,S,D]=svds(sparseWeightedMatrix,k);
    
    DS=D*S;     %the reduced space
    generateSimilaritiesSVDForQueriesOnly(DS,numberOfDocuments,numberOfQueries,listOfIssueIDs,prefixFileNameSimilarity,outputFolderLSISimilarities);
    
    fprintf('Finished saving similarities in folder %s\n',outputFolderLSISimilarities);
end















