function mainVSM(termByDocumentMatrixFileName,queriesTermByDocumentMatrixFileName,listOfIssueIDsFileName,prefixFileNameSimilarity,outputVSMSimilaritiesFolder)
%mainVSM Implementation of the Vector Space Model (VSM).
%	mainVSM computes the similarities between user queries and a corpus of
%       documents (e.g., methods, classes)
%
%	mainVSM(termByDocumentMatrixFileName,queriesTermByDocumentMatrixFileName,listOfIssueIDsFileName,outputVSMSimilaritiesFolder)
%       termByDocumentMatrixFileName - the file name of the term by document
%           matrix with numberOfDocuments rows and numberOfTerms columns
%           (the values in the matrix are separated by spaces, and each document 
%            is on its own line)
%       queriesTermByDocumentMatrixFileName - the file name of the matrix of 
%           queries with numberOfQueries rows and numberOfTerms columns (i.e., 
%           the number of columns of the matrices from
%           termByDocumentMatrixFileName and queriesTermByDocumentMatrixFileName must
%           be the same). This matrix is in the same format as the one from
%           the file termByDocumentMatrixFileName. Each row of this matrix
%           represents a vectorial representation of the query
%       listOfIssueIDsFileName - the file name containing the list of
%           issueIDs associated with the queries from the
%           queriesTermByDocumentMatrixFileName matrix. The issueID from position i is
%           associated with query i from that matrix
%       prefixFileNameSimilarity - the prefix of the VSM similarities file
%           that will embed some information about the configuration of VSM
%           (e.g., "VSM_CorpusMethods_", "VSM_CorpusClasses")
%       outputVSMSimilaritiesFolder - the folder name where the similarities
%           between each query and all the documents of the corpus will be
%           stored. One file with similarities will be generated for each
%           query and will have the name prefixFileNameSimilarity+issueID."sim". The
%           content of the file will consist of 2-tuples (one per line) in
%           the format "documentID cosineSimilarityBetweenQueryAndDocumentID"
%   example:
%       mainVSM('TestCase/termByDocumentMatrix.txt','TestCase/queriesTermByDocumentMatrix.txt','TestCase/listOfIssueIDs.txt','VSM_CorpusMethods_','TestCase/OutputVSMSimilarities/')
%
%   version 1 (2012-10-24)
%       - initial release
%
%   Copyright 2012 SEMERU Research Group http://www.cs.wm.edu/semeru/

    %size of termByDocumentMatrix should be numberOfDocuments x numberOfTerms
    termByDocumentMatrix=load(termByDocumentMatrixFileName);

    %size of queriesTermByDocumentMatrixFileName should be numberOfQueries x numberOfTerms
    queriesMatrix=load(queriesTermByDocumentMatrixFileName);
    
    %check if number of terms are the same in the matrices
    if size(termByDocumentMatrix,2)~=size(queriesMatrix,2)
        error('size(termByDocumentMatrix,2)~=size(queriesMatrix,2)');
    end;
    
	%load the issue ids; textread returns a cell, so each element (i.e.,
	%each issueID) needs to be converted to char
    listOfIssueIDs=textread(listOfIssueIDsFileName,'%s');
    
    %check if the number of queries is the same with the number of issues
    if size(queriesMatrix,1)~=size(listOfIssueIDs,1)
        error('size(queriesMatrix,1)~=size(listOfIssueIDs,1)');
    end;
    
    [weightedMatrix]=calculateTFIDF(termByDocumentMatrix);

    generateSimilaritiesVSMForQueriesOnly(weightedMatrix,queriesMatrix,listOfIssueIDs,prefixFileNameSimilarity,outputVSMSimilaritiesFolder);

    fprintf('Finished saving similarities in folder %s\n',outputVSMSimilaritiesFolder);
end